@@ -17,7 +17,7 @@ import tempfile
 import traceback
 import types
 
-import pkg_resources
+import importlib.resources
 
 from reprotest.lib import adtlog
 from reprotest.lib import adt_testbed
@@ -29,8 +29,8 @@ logger = logging.getLogger(__name__)
 VIRT_PREFIX = "autopkgtest-virt-"
 
 def get_server_path(server_name):
-    return pkg_resources.resource_filename(
-        __name__, os.path.join("virt", (VIRT_PREFIX + server_name) if server_name else ""))
+    with importlib.resources.as_file(importlib.resources.files()) as this_directory:
+        return os.path.join(str(this_directory), "virt", (VIRT_PREFIX + server_name) if server_name else "")
 
 def is_executable(parent, fn):
     path = os.path.join(parent, fn)
@@ -54,7 +54,7 @@ def get_all_servers():
 
 class Testbed(adt_testbed.Testbed):
 
-    def check_exec2(self, argv, stdout=False, kind='short', xenv=[]):
+    def check_exec2(self, argv, stdout=False, kind='install', xenv=[]):
         """Like check_exec but does not bomb on stderr, and can pass xenv."""
         (code, out, err) = self.execute(argv,
                                         stdout=(stdout and subprocess.PIPE or None),
@@ -91,7 +91,7 @@ def start_testbed(args, temp_dir, no_clean_on_error=False, host_distro=None):
         pass
     except BaseException as e:
         if no_clean_on_error:
-            logger.warn("preserving temporary files in: %s", testbed.scratch)
+            logger.warning("preserving temporary files in: %s", testbed.scratch)
             should_clean = False
         raise
     finally:
@@ -220,7 +220,17 @@ class BuildContext(collections.namedtuple('_BuildContext',
             kind='build')
         logger.info("build successful, copying artifacts")
         dist_base = os.path.join(self.testbed_dist, VSRC_DIR)
-        testbed.check_exec2(shell_copy_pattern(dist_base, self.testbed_src, artifact_pattern))
+        try:
+            testbed.check_exec2(shell_copy_pattern(dist_base, self.testbed_src, artifact_pattern))
+        except:
+            # workaround when build_path.path is not the defaults
+            os.makedirs(build.tree, exist_ok=True)
+            testbed.check_exec2(shell_copy_pattern(dist_base, build.tree, artifact_pattern))
+            # clean up artifacts and build directory
+            testbed.check_exec2(['sh', '-ec', 'cd "%s" && rm -rvf %s' %
+                                 (build.tree, artifact_pattern)])
+            testbed.check_exec2(['sh', '-ec', 'rmdir -v %s' % build.tree])
+
         # FIXME: `touch` is needed because of the FIXME in build.faketime(). we can rm it after that is fixed
         testbed.check_exec2(['sh', '-ec',
             r"""cd "{0}" && touch -d@0 . .. {1}""".format(dist_base, artifact_pattern)])
@@ -264,10 +274,10 @@ class TestbedArgs(collections.namedtuple('_TestbedArgs',
 
 
 class TestArgs(collections.namedtuple('_Test',
-    'build_command source_root artifact_pattern result_dir source_pattern no_clean_on_error diffoscope_args')):
+    'build_command source_root artifact_pattern result_dir source_pattern no_clean_on_error diffoscope_args control_build')):
     @classmethod
     def of(cls, build_command, source_root, artifact_pattern, result_dir=None,
-                source_pattern=None, no_clean_on_error=False, diffoscope_args=['diffoscope']):
+                source_pattern=None, no_clean_on_error=False, diffoscope_args=['diffoscope'], control_build=None):
         artifact_pattern = shell_syn.sanitize_globs(artifact_pattern)
         logger.debug("artifact_pattern sanitized to: %s", artifact_pattern)
 
@@ -275,7 +285,7 @@ class TestArgs(collections.namedtuple('_Test',
             source_pattern = shell_syn.sanitize_globs(source_pattern)
             logger.debug("source_pattern sanitized to: %s", source_pattern)
         return cls(build_command, source_root, artifact_pattern, result_dir,
-                   source_pattern, no_clean_on_error, diffoscope_args)
+                   source_pattern, no_clean_on_error, diffoscope_args, control_build)
 
     @coroutine
     def corun_builds(self, testbed_args):
@@ -286,7 +296,7 @@ class TestArgs(collections.namedtuple('_Test',
         .>>>     local_dist = proc.send((name, var))
         .>>>     ...
         """
-        build_command, source_root, artifact_pattern, result_dir, source_pattern, no_clean_on_error, diffoscope_args = self
+        build_command, source_root, artifact_pattern, result_dir, source_pattern, no_clean_on_error, diffoscope_args, control_build = self
         virtual_server_args, testbed_pre, testbed_init, testbed_build_pre, host_distro = testbed_args
 
         if not source_root:
@@ -347,20 +357,26 @@ class TestArgs(collections.namedtuple('_Test',
         print("Reproduction successful")
         print("=======================")
         print("No differences in %s" % self.artifact_pattern, flush=True)
-        run_or_tee(['sh', '-ec', 'find %s -type f -exec sha256sum "{}" \;' % self.artifact_pattern],
+        run_or_tee(['sh', '-ec', 'find %s -type f -exec sha256sum "{}" \\;' % self.artifact_pattern],
             'SHA256SUMS', self.result_dir,
             cwd=os.path.join(dist_control, VSRC_DIR))
 
 
 def check(test_args, testbed_args, build_variations=Variations.of(VariationSpec.default())):
     # default argument [] is safe here because we never mutate it.
-    _, _, artifact_pattern, store_dir, _, _, diffoscope_args = test_args
+    _, _, artifact_pattern, store_dir, _, _, diffoscope_args, control_build = test_args
     with empty_or_temp_dir(store_dir, "store_dir") as result_dir:
         assert store_dir == result_dir or store_dir is None
         proc = test_args._replace(result_dir=result_dir).corun_builds(testbed_args)
 
         bnames = ["control"] + ["experiment-%s" % i for i in range(1, len(build_variations))]
-        local_dists = [proc.send(nv) for nv in zip(bnames, build_variations)]
+
+        if control_build:
+            local_dists = [control_build]
+        else:
+            local_dists = [proc.send(("control", build_variations[0]))]
+
+        local_dists += [proc.send(nv) for nv in zip(bnames[1:], build_variations[1:])]
 
         retcodes = collections.OrderedDict(
             (bname, run_diff(local_dists[0], dist, diffoscope_args, store_dir))
@@ -385,7 +401,7 @@ def check(test_args, testbed_args, build_variations=Variations.of(VariationSpec.
 
 def check_auto(test_args, testbed_args, build_variations=Variations.of(VariationSpec.default())):
     # default argument [] is safe here because we never mutate it.
-    _, _, _, store_dir, _, _, diffoscope_args = test_args
+    _, _, _, store_dir, _, _, diffoscope_args, _ = test_args
     with empty_or_temp_dir(store_dir, "store_dir") as result_dir:
         assert store_dir == result_dir or store_dir is None
         proc = test_args._replace(result_dir=result_dir).corun_builds(testbed_args)
@@ -425,7 +441,7 @@ def check_auto(test_args, testbed_args, build_variations=Variations.of(Variation
 
 def check_env(test_args, testbed_args, build_variations=Variations.of(VariationSpec.default())):
     # default argument [] is safe here because we never mutate it.
-    _, _, artifact_pattern, store_dir, _, _, diffoscope_args = test_args
+    _, _, artifact_pattern, store_dir, _, _, diffoscope_args, _ = test_args
     with empty_or_temp_dir(store_dir, "store_dir") as result_dir:
         assert store_dir == result_dir or store_dir is None
         proc = test_args._replace(result_dir=result_dir).corun_builds(testbed_args)
@@ -539,7 +555,8 @@ def cli_parser():
 
     group1 = parser.add_argument_group('basic options')
     group1.add_argument('--verbosity', type=int, default=0,
-        help='An integer.  Control which messages are displayed.')
+        help='An integer. Control which messages are displayed (0: quiet ('
+             'warning/error only), 1: info, 2: debug).')
     group1.add_argument('-v', '--verbose', dest='verbosity', action='count',
         help='Like --verbosity, but given multiple times without arguments.')
     group1.add_argument('--host-distro', default=None,
@@ -599,7 +616,7 @@ def cli_parser():
         help='Give extra arguments to diffoscope when running it. Default: '
         '%(default)s. Arguments are {}-formatted with: {0} the name of each '
         'experiment run, and {1} the path of the experiment output.',
-        default=['--exclude-directory-metadata'])
+        default=['--exclude-directory-metadata=yes'])
     group2.add_argument('--diffoscope', default='diffoscope', metavar='PATH',
         help='Path to diffoscope(1). Default: %(default)s')
     group2.add_argument('--no-diffoscope', action='store_true', default=False,
@@ -635,6 +652,11 @@ def cli_parser():
     group3.add_argument('--print-sudoers', action='store_true', default=False,
         help='Print a sudoers file for passwordless operation using the given '
         '--variations, useful for user_group.available, domain_host.use_sudo.')
+    group3.add_argument('--control-build',  default=None,
+        help='Override control build with artifacts located on this path. '
+        'Allows reusing previous build as baseline.')
+    group3.add_argument('--append-build-command',  default=None, metavar='COMMANDS',
+        help='Append arguments to the build command')
 
     return parser
 
@@ -680,7 +702,7 @@ def command_line(parser, argv):
 def get_main_spec(parsed_args):
     variations = [parsed_args.variations] + parsed_args.vary
     if parsed_args.dont_vary:
-        logger.warn("--dont-vary is deprecated; use --vary=-$variation instead")
+        logger.warning("--dont-vary is deprecated; use --vary=-$variation instead")
         variations += ["-%s" % a for x in parsed_args.dont_vary for a in x.split(",")]
     return VariationSpec().extend(variations)
 
@@ -722,9 +744,9 @@ def run(argv, dry_run=None):
         elif first_arg == "auto":
             build_command = first_arg
             if parsed_args.artifact_pattern:
-                logger.warn("old CLI form `reprotest auto <source_root>` detected, "
+                logger.warning("old CLI form `reprotest auto <source_root>` detected, "
                     "setting source_root to the second argument: %s", parsed_args.artifact_pattern)
-                logger.warn("to avoid this warning, use instead `reprotest <source_root>` "
+                logger.warning("to avoid this warning, use instead `reprotest <source_root>` "
                     "or (if really necessary) `reprotest -s <source_root> auto <artifact>`")
                 source_root = parsed_args.artifact_pattern
                 parsed_args.artifact_pattern = None
@@ -734,7 +756,7 @@ def run(argv, dry_run=None):
             parts = shlex.split(first_arg)
             if len(parts) == 1:
                 if shutil.which(parts[0]) is None:
-                    logger.warn("XXX")
+                    logger.warning("XXX")
                     raise RuntimeError("Not found, neither as a file nor as a command: %s" % first_arg)
             # if len(parts) > 1 then it could be something like '( command )'
             # which is valid despite '(' not existing.
@@ -783,7 +805,7 @@ def run(argv, dry_run=None):
             specs.append(spec.extend(extra_build))
         check_func = check
     if parsed_args.min_cpus is None and not dry_run:
-        logger.warn("The control build runs on 1 CPU by default, give --min-cpus to increase this.")
+        logger.warning("The control build runs on 1 CPU by default, give --min-cpus to increase this.")
     min_cpus = parsed_args.min_cpus or 1
     build_variations = Variations.of(
         *specs,
@@ -801,10 +823,10 @@ def run(argv, dry_run=None):
         missing = [(var, tools) for var, tools in missing if tools]
         for var, tools in missing:
             if tools:
-                logger.warn("Varying '%s' requires these program(s): %s", var, ", ".join(tools))
+                logger.warning("Varying '%s' requires these program(s): %s", var, ", ".join(tools))
         if missing:
-            logger.warn("Your build will probably fail, either install them or disable the variations.")
-            logger.warn("(From a system package manager, simply install the 'optional' or 'recommended' "
+            logger.warning("Your build will probably fail, either install them or disable the variations.")
+            logger.warning("(From a system package manager, simply install the 'optional' or 'recommended' "
                          "dependencies of reprotest.)")
 
     # Remaining args
@@ -814,8 +836,15 @@ def run(argv, dry_run=None):
     diffoscope = parsed_args.diffoscope
     if parsed_args.no_diffoscope:
         diffoscope_args = None
-    else:
+    elif shutil.which('diffoscope'):
         diffoscope_args = [diffoscope] + diffoscope_args
+    else:
+        logger.warning("diffoscope not available, falling back to regular diff")
+        diffoscope_args = None
+    control_build = parsed_args.control_build
+    append_build_command = parsed_args.append_build_command
+    if append_build_command :
+        build_command = ' '.join([build_command, append_build_command])
 
     if not artifact_pattern:
         print("No <artifact> to test for differences provided. See --help for options.")
@@ -823,7 +852,7 @@ def run(argv, dry_run=None):
 
     testbed_args = TestbedArgs.of(virtual_server_args, testbed_pre, testbed_init, testbed_build_pre, host_distro)
     test_args = TestArgs.of(build_command, source_root, artifact_pattern, store_dir,
-                            source_pattern, no_clean_on_error, diffoscope_args)
+                            source_pattern, no_clean_on_error, diffoscope_args, control_build)
 
     check_args = (test_args, testbed_args, build_variations)
     if dry_run:
