@@ -4,10 +4,10 @@
 import contextlib
 import logging
 import os
+import pytest
 import subprocess
 import sys
 
-import pytest
 import reprotest
 from reprotest.build import VariationSpec, Variations, VARIATIONS
 
@@ -20,6 +20,7 @@ if REPROTEST_TEST_DONTVARY:
 
 TEST_VARIATIONS = frozenset(VARIATIONS.keys()) - frozenset(REPROTEST_TEST_DONTVARY)
 
+
 def check_reproducibility(command, virtual_server, reproducible):
     result = reprotest.check(
         reprotest.TestArgs.of(command, 'tests', 'artifact'),
@@ -27,6 +28,7 @@ def check_reproducibility(command, virtual_server, reproducible):
         Variations.of(VariationSpec.default(TEST_VARIATIONS)))
     assert result == reproducible
 
+
 def check_command_line(command_line, code=None):
     try:
         retcode = 0
@@ -41,6 +43,7 @@ def check_command_line(command_line, code=None):
         else:
             assert(retcode in code)
 
+
 @pytest.fixture(scope='module', params=REPROTEST_TEST_SERVERS)
 def virtual_server(request):
     if request.param == 'null':
@@ -52,11 +55,6 @@ def virtual_server(request):
     else:
         raise ValueError(request.param)
 
-def test_simple_builds(virtual_server):
-    check_reproducibility('python3 mock_build.py', virtual_server, True)
-    with pytest.raises(Exception):
-        check_reproducibility('python3 mock_failure.py', virtual_server)
-    check_reproducibility('python3 mock_build.py irreproducible', virtual_server, False)
 
 @contextlib.contextmanager
 def setup_logging(debug):
@@ -82,6 +80,14 @@ def setup_logging(debug):
         logger.removeHandler(ch)
         logger.setLevel(oldLevel)
 
+
+def test_simple_builds(virtual_server):
+    check_reproducibility('python3 mock_build.py', virtual_server, True)
+    with pytest.raises(Exception):
+        check_reproducibility('python3 mock_failure.py', virtual_server)
+    check_reproducibility('python3 mock_build.py irreproducible', virtual_server, False)
+
+
 # TODO: test all variations that we support
 @pytest.mark.parametrize('captures', list(VARIATIONS.keys()))
 def test_variations(virtual_server, captures):
@@ -89,6 +95,7 @@ def test_variations(virtual_server, captures):
     with setup_logging(False):
         check_reproducibility('python3 mock_build.py ' + captures, virtual_server, expected)
 
+
 @pytest.mark.need_builddeps
 def test_self_build(virtual_server):
     # at time of writing (2016-09-23) these are not expected to reproduce;
@@ -98,6 +105,7 @@ def test_self_build(virtual_server):
     assert(1 == subprocess.call(REPROTEST + ['python3 setup.py bdist', 'dist/*.tar.gz'] + virtual_server))
     assert(1 == subprocess.call(REPROTEST + ['python3 setup.py sdist', 'dist/*.tar.gz'] + virtual_server))
 
+
 def test_command_lines():
     test_args, _, _ = check_command_line(".".split(), 0)
     assert test_args.artifact_pattern is not None
@@ -128,7 +136,8 @@ def test_command_lines():
     _, testbed_args, _ = check_command_line(". -- schroot unstable-amd64-sbuild".split(), 0)
     assert testbed_args.virtual_server_args == ['schroot', 'unstable-amd64-sbuild']
 
-# TODO: don't call it if we don't have debian/, e.g. for other distros
+
+@pytest.mark.debian
 @pytest.mark.need_builddeps
 def test_debian_build(virtual_server):
     # This is a bit dirty though it works - when building the debian package,
